// Based on DLLInject demonstratation by Rezmond( http://www.torry.net/authorsmore.php?id=3987 )
unit RemoteDLLExec;

interface

uses
  Windows;

type
  // Params for GetProcAddress call
  PCallGetProcAddressParams = ^TCallGetProcAddressParams;
  TCallGetProcAddressParams = record
    hDLL: HMODULE;
    pGetProcAddress: Pointer;
    // Shouldn't send a pointer to own memory space to another so we copy the whole array
    ProcName: Array[0..MAX_PATH] of AnsiChar;
  end;

// Initializes a new Params structure
function NewCallGetProcAddressParams(const aDLLHandle: HMODULE; const aProcName: string): PCallGetProcAddressParams;

// Calls GetProcAddress, stdcall to allow injection.
// This method can be executed by CreateRemoteThread as does LoadLibrary
function CallGetProcAddress(aParams: PCallGetProcAddressParams): Pointer; stdcall;

// Calls a GetProcAddress from a remote process space
function RemoteCallGetProcAddress(const aProcHandle, aDLLHandle: HMODULE; const aProcName: string): Pointer;

implementation

uses
  SysUtils, CodeSize;

function NewCallGetProcAddressParams(const aDLLHandle: HMODULE; const aProcName: string): PCallGetProcAddressParams;
begin
  // blank initialization
  New(Result);
  FillChar(Result^, SizeOf(TCallGetProcAddressParams), 0);

  Result.hDLL:= aDLLHandle; // DLL Handle in remote process space
  Result.pGetProcAddress:= GetProcAddress(LoadLibrary('Kernel32.dll'), 'GetProcAddress');
  // Copy the string to own array
  StrLCopy(Result.ProcName, PAnsiChar(AnsiString(aProcName)), MAX_PATH-1);
end;

function CallGetProcAddress(aParams: PCallGetProcAddressParams): Pointer; stdcall;
var
  hDLL: HMODULE;
  ProcName: PAnsiChar;
  GetProcAddressPtr, ProcAddr: Pointer;

begin
  // convert to local vars for ease of reading
  hDLL:= aParams^.hDLL;
  ProcName:= @aParams^.ProcName;
  GetProcAddressPtr:= aParams.pGetProcAddress;
  asm
    // stdcall methods receibe parameters from stack in right-to-left order
    push ProcName;
    push hDLL;
    call GetProcAddressPtr;
    mov DWORD PTR[ProcAddr], eax;
  end;
  // return GetProcAdress's result
  Result:= ProcAddr;
end;

function RemoteCallGetProcAddress(const aProcHandle, aDLLHandle: HMODULE; const aProcName: string): Pointer;
var
  CallGetProcAddressSize,
  CallGetProcAddressParamsSize,
  BytesWritten: Cardinal;

  RemoteCallGetProcAddress,
  RemoteCallGetProcAddressParams: Pointer;

  Params: PCallGetProcAddressParams;

  ThreadHandle: THandle;
  ThreadID: Cardinal;

begin
  // Get Size of CallGetProcAddress procedure
  CallGetProcAddressSize:= SizeOfProc(@CallGetProcAddress);
  // Allocate memory on remote process to copy CallGetProcAddress
  // Has to have EXECUTE permision otherwise it will raise an AccessViolation
  RemoteCallGetProcAddress:= VirtualAllocEx(aProcHandle, nil, CallGetProcAddressSize, MEM_COMMIT, PAGE_EXECUTE_READWRITE);
  try
    // Copy procedure
    WriteProcessMemory(aProcHandle, RemoteCallGetProcAddress, @CallGetProcAddress, CallGetProcAddressSize, BytesWritten);

    // Initialize a new Params Structure
    Params:= NewCallGetProcAddressParams(aDLLHandle, aProcName);
    CallGetProcAddressParamsSize:= SizeOf(TCallGetProcAddressParams);
    RemoteCallGetProcAddressParams:= VirtualAllocEx(aProcHandle, nil, CallGetProcAddressParamsSize, MEM_COMMIT, PAGE_READWRITE);
    try
      WriteProcessMemory(aProcHandle, RemoteCallGetProcAddressParams, Params, CallGetProcAddressParamsSize, BytesWritten);
      Dispose(Params);

      ThreadHandle:= CreateRemoteThread(aProcHandle, nil, 0, RemoteCallGetProcAddress, RemoteCallGetProcAddressParams, 0, ThreadID);
      WaitForSingleObject(ThreadHandle, INFINITE);
      GetExitCodeThread(ThreadHandle, Cardinal(Result));
      CloseHandle(ThreadHandle);
    finally
      VirtualFreeEx(aProcHandle, RemoteCallGetProcAddressParams, 0, MEM_RELEASE);
    end;
  finally
    VirtualFreeEx(aProcHandle, RemoteCallGetProcAddress, 0, MEM_RELEASE);
  end;

end;

end.
